using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[ExecuteInEditMode]
public class RadialJedi : Jedi
{
	private AudioSource audioSource;

    private float _radius = 1f;
    [SerializeField] public float radius {
        get { return _radius; }
        set { _radius = value;
            UpdateRadiusDependencies();
        }
    }

    [SerializeField] private CircleCollider2D colliderRef;
    [SerializeField] private Transform radiusArtRef;


    private void Start()
	{
		pulseImage.gameObject.SetActive(bPulsing);

		audioSource = GetComponent<AudioSource>();
	}

	private void Update()
	{
		if(bPulsing)
		{
			float scale0 = 0,scale1 = 0;

			if(accelPercentage >= 0)
			{
				scale0 = 16f;
				scale1 = 2f;
			}
			else
			{
				scale0 = 2f;
				scale1 = 16f;
			}

			pulseImage.localScale = Vector3.Lerp(new Vector3(scale0,scale0,scale0),new Vector3(scale1,scale1,scale1),(Mathf.Sin(Time.time * pulseSpeed)/2f) + 0.5f);
		}
	}
	
	private void OnTriggerStay2D(Collider2D collider)
	{
		Drone drone = collider.GetComponent<Drone>();

		if(drone != null)
		{
			ApplyForceToDrone(drone,transform.position - drone.transform.position);

			if(!audioSource.isPlaying)
				audioSource.PlayOneShot(audioSource.clip);
		}
	}

    private void UpdateRadiusDependencies () {
        if (colliderRef) {
            colliderRef.radius = radius;
        }
        if (radiusArtRef) {
            radiusArtRef.localScale = Vector3.one * radius * 2;
        }
    }
}
