using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Banana : MonoBehaviour {

	private AudioSource audioSource;
	public AudioClip impact,activation;

    [SerializeField] private float initialForce = 1f;
    private Rigidbody2D rb2d;

    [SerializeField] private float radius;

    [SerializeField] private Material activeMat;
    [SerializeField] private Material inactiveMat;

    [SerializeField] private bool active;
    [SerializeField] private float minTimeBeforeActive;
    [SerializeField] private float maxTimeBeforeActive;


    private float nextActiveTime;

    [Header("GameObject Refs")]
    [SerializeField] private ParticleSystem sparkle;
    [SerializeField] private ParticleSystem pulseWave;
    [SerializeField] private ParticleSystem activateWave;
    [SerializeField] private ParticleSystem pickupWave;

    private SwarmTarget[] players;

    void OnEnable() {

        // Get reference to rigidbody and collider trigger
        rb2d = GetComponent<Rigidbody2D>();

        players = FindObjectsOfType<SwarmTarget>();
        
    }


	// Use this for initialization
	void Start () {

        rb2d.velocity = ( new Vector2(Random.Range(-1f, 1f), Random.Range(-1f, 1f)) * initialForce);

        // Start inactive
        Deactivate();

		audioSource = GetComponent<AudioSource>();
	}
	
	// Update is called once per frame
	void Update () {

		if (Time.time >= nextActiveTime && !active) {
            Activate();
        }

        if ( active ) {
            float distance;
            for ( int i = 0; i < players.Length; i++) {
                distance = Vector3.Distance(players[i].transform.position, transform.position);

                if (distance < radius * transform.localScale.x ) {
                    ActivateBeastMode(players[i]);
                    break;
                }

            }
        }
        
	}


    void Activate() {
        active = true;
        GetComponent<Renderer>().material = activeMat;
        // TODO: Effects!
        activateWave.Play();
        sparkle.Play();
        pulseWave.Play();

		audioSource.PlayOneShot(activation);
    }


    void Deactivate() {
        // Set Inactive
        active = false;
        // Get next activate time
        nextActiveTime = SetNextActiveTime();

        // Change Material
        GetComponent<Renderer>().material = inactiveMat;
        // Stop Effects
        activateWave.Stop();
        pickupWave.Stop();
        sparkle.Stop();
        pulseWave.Stop();
        
    }


    private void ActivateBeastMode( SwarmTarget player ) {

        active = false;

        if ( player.GetComponent<SwarmTarget>() ) {
            pickupWave.Play();
            // TODO: Get an array of the drones in the collided player's swarm
            foreach ( Drone d in DroneSpawner.allDrones ) {
                if (d != null && d.swarmTarget == player) {
                    d.BeastModeOn();
                }
            }
            Deactivate();
        }
    }


    private float SetNextActiveTime() {
        return Time.time + Random.Range(minTimeBeforeActive, maxTimeBeforeActive);
    }


	private void OnCollisionEnter2D(Collision2D collision)
	{
		if(!audioSource.isPlaying && !IsInvoking("WaitSound"))
		{
			audioSource.PlayOneShot(impact);

			Invoke("WaitSound",0.3f);
		}
	}


	public void WaitSound()
	{
	}

    void OnDrawGizmosSelected () {
        Gizmos.color = Color.yellow;
        Gizmos.DrawSphere(transform.position, radius * transform.localScale.x);
    }
}
