using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Drone : MonoBehaviour
{

    public static int populationCount = 0;

	private bool bBeastMode;

	// Set in editor
	public GameObject explosionPrefabRed,explosionPrefabBlue,explosionPrefabWhite;
	public ParticleSystem beastModePSysPrefab;

	public float attractionAccel;
	public float brakingAccel;
	public float randomAccel;
	public float timeUntilNeutral;
	public float beastModeAccelMultiplier;

	public float beastModeResetTimer = 10f;

	public SwarmTarget swarmTarget { get; set; }
	public Rigidbody2D rb { get; private set; }

	public PlayerNumber playerNum;

	private Renderer renderer;

	private float timeAtNeutralizeStart;
	private Color originalColor;

    void Enable () {
        populationCount += 1;
    }

	// Use this for initialization
	void Start () {
		rb = GetComponent<Rigidbody2D>();
		renderer = GetComponentInChildren<Renderer>();
	}
	
	// Update is called once per frame
	void Update () {
	
	}

	void FixedUpdate() {
		if(swarmTarget == null)
			return;

		// The drone has a constant force towards the target
		Vector2 attractionForce = swarmTarget.transform.position - transform.position;
		attractionForce = attractionForce.normalized * attractionAccel;
		if (bBeastMode) {
			attractionForce *= beastModeAccelMultiplier;
		}

		// The drone will brake when it's not facing the right direction
		float angle = Vector2.Angle(attractionForce, rb.velocity);
		Vector2 brakeForce = -rb.velocity.normalized * Mathf.Sin(angle * Mathf.Deg2Rad) * brakingAccel;

		// Add a small random element to make things more lively
		Vector2 randomForce = Random.insideUnitCircle.normalized * randomAccel;

		rb.AddForce((attractionForce + brakeForce + randomForce) * rb.mass);
	}

	public void BeastModeOn()
	{
		if(bBeastMode || swarmTarget == null)
			return;
		
		bBeastMode = true;

		rb.mass *= 15f;

		transform.localScale *= 2;

		ParticleSystem test = 
		Instantiate(beastModePSysPrefab,new Vector3(transform.position.x,transform.position.y + 1f,transform.position.z - 1.5f),beastModePSysPrefab.transform.rotation,transform).GetComponent<ParticleSystem>();

		float brighten = 1.5f;

		test.startColor = new Color(swarmTarget.player.playerColor.r * brighten,swarmTarget.player.playerColor.g * brighten,swarmTarget.player.playerColor.b * brighten,1);

		swarmTarget.audioSource.PlayOneShot(swarmTarget.beastMode);

		//swarmTarget.bExplodeSoundReady = true;

		Invoke("Explode",beastModeResetTimer);
//		swarmTarget.Invoke("ExplodeSoundReady",11f);
	}

	public void BeastModeOff()
	{
		if(!bBeastMode)
			return;

		bBeastMode = false;

		rb.mass /= 15f;

		transform.localScale /= 2;
	}

	private void OnCollisionEnter2D(Collision2D collision)
	{
		if(!bBeastMode || playerNum == PlayerNumber.None)
			return;

		Drone drone = collision.collider.GetComponent<Drone>();

		if(drone != null && drone.playerNum != playerNum)
		{
			drone.Convert(playerNum);
		}
	}

	// Muahahahaha
	public void Explode() {
//
//		if (playerNum == PlayerNumber.One) {
//			Instantiate (explosionPrefabRed, transform.position, transform.rotation);
//		} else if(playerNum == PlayerNumber.Two){
//			Instantiate (explosionPrefabBlue, transform.position, transform.rotation);
//		}
//		else
//			Instantiate (explosionPrefabWhite, transform.position, transform.rotation);
//
//		SwarmTarget tempSwarm = swarmTarget;
//
//		if(tempSwarm == null)
//			tempSwarm = SwarmTarget.allSwarmTargets[0];
//
//		if(tempSwarm.bExplodeSoundReady)
//		{
//			tempSwarm.audioSource.PlayOneShot(tempSwarm.explode);
//
//			tempSwarm.bExplodeSoundReady = false;
//		}
//
//		Destroy(gameObject);

		BeastModeOff();
	}

	public void Convert(PlayerNumber newPlayerNum) {

		if(playerNum != PlayerNumber.None && newPlayerNum != PlayerNumber.None && !swarmTarget.audioSource.isPlaying)
			swarmTarget.audioSource.PlayOneShot(swarmTarget.conversionSounds[Random.Range(0,swarmTarget.conversionSounds.Count)]);

		playerNum = newPlayerNum;

		if(playerNum != PlayerNumber.None)
		{
			for(int i=0; i<SwarmTarget.allSwarmTargets.Count; i++)
			{
				HumanInput player = SwarmTarget.allSwarmTargets[i].player;

				if(player.playerNumber == playerNum)
				{
					swarmTarget = SwarmTarget.allSwarmTargets[i];

					renderer.material.color = player.playerColor;
				}
			}
		}
		else
		{
			swarmTarget = null;

			BeastModeOff();

			GetComponentInChildren<Renderer>().material.color = Color.white;
		}
	}

	public void TimerToPlayerZeroStart() {
		originalColor = renderer.material.color;
		timeAtNeutralizeStart = Time.time;
		TimerToPlayerZero();
	}

	private void TimerToPlayerZero()
	{
		if(!IsInvoking("ConvertToPlayerZero"))
			Invoke("ConvertToPlayerZero", timeUntilNeutral);

		if(!IsInvoking("ColorShift"))
			InvokeRepeating("ColorShift",0,0.01f);
	}

	public void ConvertToPlayerZero()
	{
		Convert(PlayerNumber.None);

		CancelPlayerZeroConvert();
	}

	public void CancelPlayerZeroConvert()
	{
		CancelInvoke("ConvertToPlayerZero");
		CancelInvoke("ColorShift");

		if(swarmTarget != null)
			renderer.material.color = swarmTarget.player.playerColor;
	}

	public void ColorShift()
	{
		float percent = (Time.time - timeAtNeutralizeStart) / timeUntilNeutral;
		renderer.material.color = new Color(originalColor.r + percent, originalColor.g + percent, originalColor.b + percent, 1);
	}


    void OnDisable() {
        populationCount -= 1;
		DroneSpawner.allDrones.Remove(this);
    }
}
