using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class DroneSpawner : MonoBehaviour {


    /// <summary>
    /// The global maximum number of drones we can have in action at any given time.
    /// </summary>
    protected static int totalPopulation = 300;

    public static HashSet<Drone> allDrones = new HashSet<Drone>();

    /// <summary>
    /// Drone Prefab Reference
    /// </summary>
    public GameObject dronePrefab; //crowd who are spawned
    /// <summary>
    /// Number of drones spawned per second.
    /// </summary>
    public float spawnRate = 0.025f;

    // Set during startup
    [HideInInspector]
    public HumanInput player;

    private Transform droneContainer;

    void OnEnable() {
        // get the drone container to make me sane
        GameObject dc = new GameObject("Drone Container");
        if (dc != null ) {
            droneContainer = dc.transform;
        } else {
            Debug.LogError("You need an object tagged as 'DroneContainer' to parent your drones to.");
        }
        allDrones.Clear();
    }


    public bool spawning { get; private set; }
    public void SpawnWave(int waveSize) {
        spawning = true;
        StartCoroutine(SpawnNewWave(waveSize));
    }
    

    private void SpawnDrone() {
        //instantiates slime
        GameObject newDrone = Instantiate(
            dronePrefab,
			this.transform.position + (Vector3)Random.insideUnitCircle,
            Quaternion.identity
            );
        newDrone.transform.SetParent(droneContainer, true);

		Drone drone = newDrone.GetComponent<Drone>();
        drone.Convert(PlayerNumber.None);
        allDrones.Add(drone);

        // Assign the drone to the correct player target and give it that player's color.
//        newDrone.GetComponent<Drone>().swarmTarget = player.swarmTarget;
//        newDrone.GetComponentInChildren<Renderer>().material.color = player.playerColor;
    }


    IEnumerator SpawnNewWave( int waveSize ) {
        int waveDroneCount = 0;
        while ( waveDroneCount <= waveSize ) {

            SpawnDrone();

            waveDroneCount++;
            //	print ("cour run");
            yield return new WaitForSeconds(spawnRate);

        }
        spawning = false;
        yield return null;
    }
}
