using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class GameManager : MonoBehaviour {

	private AudioSource audioSource;
	public AudioClip score,gameOver;

    [Header("Spawn Values")]
    [SerializeField] private int numSpawnedPerWave = 30;
    [SerializeField] private int numSpawnVariance = 5;
    [SerializeField] private float delayBtwnWaves = 5;
    [SerializeField] private int maxPopulation = 300;
    private float nextWaveTime;

	public Text scoreText;
	public Text timerText;
    public Text gameOverText;
    public RectTransform gameOverScreen;
    public float gameLengthSec = 120;

	public int winScore = 200;
    private int p1Score = 0;
	private int p2Score = 0;
    private float gameTimer;
    private DroneSpawner[] spawners;
    public bool playing { get; private set; }

    [SerializeField] private bool doSpawn = false;

    void Start() {
        SetNextSpawnTime();
        gameTimer = gameLengthSec;
        spawners = FindObjectsOfType<DroneSpawner>();
		scoreText.text = string.Format("<color=red>{0}</color> <b>:</b> <color=blue>{1}</color>", p1Score, p2Score);  
        playing = true;
        if (gameOverScreen != null) {
            gameOverScreen.gameObject.SetActive(false);    
        }
        timerText.gameObject.SetActive(false);
		audioSource = GetComponent<AudioSource>();
    }

    void Update() {

        // If it's time to spawn guys, spawn some number based on set values.
        if (spawners.Length > 0 && Time.time >= nextWaveTime && Drone.populationCount < maxPopulation ) {
            if (doSpawn) spawners[Random.Range(0, spawners.Length)].SpawnWave(
                Random.Range(numSpawnedPerWave - numSpawnVariance, numSpawnedPerWave + numSpawnVariance)
            );
            SetNextSpawnTime();
        } else if (spawners.Length <= 0) {
            Debug.LogError("You don't have any spawners assigned to your Game Manager");
        }

//        gameTimer -= Time.deltaTime;
//        if (gameTimer <= 0 && playing) {
//            GameOver();
//        }
//        timerText.text = string.Format("{0}.{1}", (int)gameTimer, (int)(gameTimer * 10) % 10);
    }

    void SetNextSpawnTime() {
        nextWaveTime = Time.time + delayBtwnWaves;
    }

	public void updateScore(PlayerNumber scoringPlayer){
		if (scoringPlayer == PlayerNumber.One) {
			p1Score++;
		} else if (scoringPlayer == PlayerNumber.Two) {
			p2Score++;
		} else {
			Debug.LogError("Attempted to update score for invalid player");
		}
		scoreText.text = string.Format("<color=red>{0}</color> <b>:</b> <color=blue>{1}</color>", p1Score, p2Score);

		if(p1Score >= winScore || p2Score >= winScore)
			GameOver();

		if(!IsInvoking("SoundWait"))
		{
			audioSource.PlayOneShot(score);

			Invoke("SoundWait",0.2f);
		}
	}

	public void SoundWait()
	{
		
	}

    public void RestartGame() {
        Debug.Log("Hello world");
        Application.LoadLevel(UnityEngine.SceneManagement.SceneManager.GetActiveScene().buildIndex);
    }

    private void GameOver() {
        gameTimer = 0;
        playing = false;
        if (p1Score > p2Score) {
            gameOverText.text = string.Format("GAME OVER\nRED PLAYER WINS");
        } else if (p2Score > p1Score) {
            gameOverText.text = string.Format("GAME OVER\nBLUE PLAYER WINS");
        } else {
            gameOverText.text = string.Format("GAME OVER\nTIE GAME");
        }
        gameOverScreen.gameObject.SetActive(true);

		audioSource.PlayOneShot(gameOver);
    }
}
